#include "c:\bc\gvlib\collect.h"
#include <process.h>

static const maxCollectionSize=0x4fff;

template <class Type> CCollection<Type>::CCollection( int aLimit, int aDelta ) :
	count( 0 ),
	items( 0 ),
	limit( 0 ),
	delta( aDelta ),
	shouldDelete( True )
	{
	setLimit( aLimit );
	}

template <class Type> CCollection<Type>::~CCollection()
	{
	}

template <class Type> void CCollection<Type>::shutDown()
	{
	if( shouldDelete )
		freeAll();
	setLimit(0);
	AnObject::shutDown();
	}

template <class Type> Type * Collection<Type>::at( int index )
	{
	return items[index];
	}

template <class Type> void CCollection<Type>::atRemove( int index )
	{
	if( index >= count )
		error(1,0);

	count--;
	memmove( &items[index], &items[index+1], (count-index)*sizeof(Type *) );
	}

template <class Type> void CCollection<Type>::atFree( int index )
	{
	Type *item = at( index );
	atRemove( index );
	freeItem( item );
	}

template <class Type> void CCollection<Type>::atInsert(int index, Type *item)
	{
	if( index < 0 )
		error(1,0);
	if( count == limit )
		setLimit(count + delta);

	memmove( &items[index+1], &items[index], (count-index)*sizeof(Type *) );
	count++;

	items[index] = item;
	}

template <class Type> void CCollection<Type>::atPut( int index, Type *item )
	{
	if( index >= count )
		error(1,0);

	items[index] = item;
	}

template <class Type> void CCollection<Type>::remove( Type *item )
	{
	atRemove( indexOf(item) );
	}

template <class Type> void CCollection<Type>::removeAll()
	{
	count = 0;
	}

template <class Type> void CCollection<Type>::error( int code, int )
	{
	exit(212 - code);
	}

template <class Type> Type * CCollection<Type>::firstThat(Bool (*Test)(Type *,void *),void *arg )
	{
	for( int i = 0; i < count; i++ )
		{
		if( Test( items[i], arg ) == True )
			return items[i];
		}
	return 0;
	}

template <class Type> Type * CCollection<Type>::lastThat( Bool (*Test)(Type *,void *), void *arg )
	{
	for( int i = count; i > 0; i-- )
		{
		if( Test( items[i-1], arg ) == True )
			return items[i-1];
		}
	return 0;
	}

template <class Type> void CCollection<Type>::forEach( void (*action)(Type *,void *), void *arg )
	{
	for( int i = 0; i < count; i++ )
		action( items[i], arg );
	}

template <class Type> void CCollection<Type>::free( Type *item )
	{
	remove( item );
	freeItem( item );
	}

template <class Type> void CCollection<Type>::freeAll()
	{
	for( int i =  0; i < count; i++ )
		freeItem( at(i) );
	count = 0;
	}

template <class Type> void CCollection<Type>::freeItem( Type *item )
	{
	delete item;
	}

#pragma warn -rvl
template <class Type> int CCollection<Type>::indexOf(Type *item)
	{
	for( int i = 0; i < count; i++ )
		if( item == items[i] )
			return i;

	return -1;
	}
#pragma warn .rvl

template <class Type> int CCollection<Type>::insert( Type *item )
	{
	int loc = count;
	atInsert( count, item );
	return loc;
	}

template <class Type> void CCollection<Type>::pack()
	{
	Type **curDst = items;
	Type **curSrc = items;
	Type **last = items + count;
	while( curSrc < last )
		{
		if( *curSrc != 0 )
			*curDst++ = *curSrc;
		*curSrc++;
		}
	}

template <class Type> void CCollection<Type>::setLimit(int aLimit)
	{
	if( aLimit < count )
		aLimit =  count;
	if( aLimit > maxCCollectionSize)
		aLimit = maxCCollectionSize;
	if( aLimit != limit )
		{
		Type **aItems;
		if (aLimit == 0 )
			aItems = 0;
		else
			{
			aItems = new Type *[aLimit];
			if( count !=  0 )
				memcpy( aItems, items, count*sizeof(Type *) );
			}
		delete items;
		items =  aItems;
		limit =  aLimit;
		}
	}

template <class Type>
int SortedCCollection<Type>::insert(Type * t)
     {
     if(getCount()==0) {  return CCollection<Type>::insert(t); }
     int b=getCount();
     for(int i=0;i<b;i++)
         {
         int c=compare(at(i),t);
         if(c>=0)
              if((c!=0)||(enableDuplicates))
                  {
		  CCollection<Type>::atInsert(i,t);
                  return i;
                  }
              else return 0;
         }
     return CCollection<Type>::insert(t);
     }
